@@ -1,4 +1,4 @@
-# Arch Perl library, Copyright (C) 2004 Mikhael Goikhman
+# Arch Perl library, Copyright (C) 2004-2005 Mikhael Goikhman
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,32 +19,22 @@ use strict;
 
 package Arch::Util;
 
+# import 2 functions for backward compatibility only; remove after summer 2005
+use Arch::Backend qw(arch_backend is_baz);
+
 use Exporter;
-use vars qw(@ISA @EXPORT_OK $ARCH_BACKEND);
+use vars qw(@ISA @EXPORT_OK);
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(
 	arch_backend is_baz
 	run_pipe_from run_cmd run_tla
+	is_tla_functional
 	load_file save_file
 	copy_dir remove_dir setup_config_dir
-	standardize_date date2daysago parse_creator_email adjacent_revision
-	_parse_revision_descs
+	standardize_date date2daysago date2age
+	parse_creator_email adjacent_revision _parse_revision_descs
 );
 
-sub arch_backend (;$) {
-	$ARCH_BACKEND = shift if @_;
-	$ARCH_BACKEND ||= $ENV{TLA} || $ENV{ARCH_BACKEND} || "tla";
-	return $ARCH_BACKEND;
-}
-
-sub is_baz () {
-	return scalar(($ARCH_BACKEND || "") =~ m!(^|/)baz[^/]*$!);
-}
-
-BEGIN {
-	arch_backend(undef);
-}
-
 sub run_pipe_from (@) {
 	my $arg0 = shift || die;
 	my @args = (split(' ', $arg0), @_);
@@ -56,7 +46,7 @@ sub run_pipe_from (@) {
 
 	# perl-5.005 does not pass compilation without "eval"...
 	my $pipe_success = $] >= 5.006?
-		eval qq{ open(PIPE, '-|', \@args) }: open(PIPE, "$args[0]|");
+		eval qq{ no warnings; open(PIPE, '-|', \@args) }: open(PIPE, "$args[0]|");
 	die "Can't start (@args): $!\n" unless $pipe_success;
 	return \*PIPE;
 }
@@ -71,15 +61,19 @@ sub run_cmd (@) {
 	my @lines = <$pipe>;
 	close($pipe);
 	chomp @lines if wantarray;
-	return wantarray? @lines: $lines[0];
+	return wantarray? @lines: $lines[0] || "";
 }
 
 sub run_tla (@) {
 	my $arg1 = shift || die;
-	unshift @_, $ARCH_BACKEND, split(' ', $arg1);
+	unshift @_, $Arch::Backend::EXE, split(' ', $arg1);
 	goto \&run_cmd;
 }
 
+sub is_tla_functional () {
+	eval { run_tla("help --help") } ? 1 : 0;
+}
+
 sub load_file ($;$) {
 	my $file_name = shift;
 	my $content_ref = shift;
@@ -90,9 +84,11 @@ sub load_file ($;$) {
 	my $content = <FILE>;
 	close(FILE) or die "Can't close $file_name in load: $!\n";
 	if ($content_ref) {
-		$content_ref = \$content if ref($content_ref) eq 'SCALAR';
-		@$content_ref = map { chomp; $_ } split(/\r?\n/, $content)
-			if ref($content_ref) eq 'ARRAY';
+		$$content_ref = $content if ref($content_ref) eq 'SCALAR';
+		if (ref($content_ref) eq 'ARRAY') {
+			$content =~ s/\r?\n$//;
+			@$content_ref = map { chomp; $_ } split(/\r?\n/, $content, -1);
+		}
 	}
 	return defined wantarray? $content: undef;
 }
@@ -118,8 +114,9 @@ sub copy_dir ($$) {
 }
 
 sub remove_dir (@) {
-	return unless @_;
-	my $out = run_cmd("/bin/rm -rf", @_);
+	my @dirs = grep { $_ } @_;
+	return unless @dirs;
+	my $out = run_cmd("/bin/rm -rf", @dirs);
 	warn $out if $out;
 }
 
@@ -156,14 +153,15 @@ sub standardize_date ($) {
 	return $date;
 }
 
-# return (creator_name, creator_email)
+# return (creator_name, creator_email, creator_username)
 sub parse_creator_email ($) {
 	my $creator = shift;
 	my $email = 'no@email.defined';
-	if ($creator =~ /^(.*?)\s*<(.*)>$/) {
-		($creator, $email) = ($1, $2);
+	my $username = "_none_";
+	if ($creator =~ /^(.*?)\s*<((?:(.+?)@)?.*)>$/) {
+		($creator, $email, $username) = ($1, $2, $3);
 	}
-	return ($creator, $email);
+	return ($creator, $email, $username);
 }
 
 sub adjacent_revision ($$) {
@@ -185,7 +183,7 @@ sub adjacent_revision ($$) {
 sub date2daysago ($) {
 	my $date_str = shift;
 
-	return -999 unless $date_str =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2}) ([^\s]+)/;
+	return -10000 unless $date_str =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2}) ([^\s]+)/;
 
 	# timezone is not taken in account...
 	require Time::Local;
@@ -196,6 +194,20 @@ sub date2daysago ($) {
 	return ($daysago, $time, $7);
 }
 
+sub date2age ($) {
+	my $daysago = date2daysago($_[0]);
+	return "bad-date" if $daysago <= -10000;
+
+	my ($sign, $days) = $daysago =~ /^(-?)(.*)$/;
+	my $str =
+		$days ==   1? "1 day":
+		$days <=  33? "$days days":
+		$days <=  59? int($days / 7 + 0.5) . " weeks":
+		$days <= 550? int($days / 30.42 + 0.5) . " months":
+		int($days / 365.25 + 0.5) . " years";
+	return "$sign$str";
+}
+
 # gets tla lines with undef meaning the delimiter for revisions;
 # intended for parsing of "abrowse --desc" and "logs --cDs"
 sub _parse_revision_descs ($$) {
@@ -220,18 +232,21 @@ sub _parse_revision_descs ($$) {
 		my ($date, $creator) = $line2 =~ /^(.+?)\s{6}(.*)/
 			or die "Unexpected output of tla, subline 2:\n\t$line2\n";
 		$date = standardize_date($date);
+		my $age = date2age($date);
 
 		my @version_part;
 		push @version_part, 'version', $1 if $name =~ s/^(.*)--(.*)/$2/;
 
-		my ($creator_name, $creator_email) = parse_creator_email($creator);
+		my ($creator1, $email, $username) = parse_creator_email($creator);
 		push @revision_descs, {
-			name    => $name,
-			summary => $summary,
-			creator => $creator_name,
-			email   => $creator_email,
-			date    => $date,
-			kind    => $kind,
+			name     => $name,
+			summary  => $summary,
+			creator  => $creator1,
+			email    => $email,
+			username => $username,
+			date     => $date,
+			age      => $age,
+			kind     => $kind,
 			@version_part,
 		};
 	}
@@ -280,6 +295,7 @@ B<remove_dir>,
 B<setup_config_dir>,
 B<standardize_date>,
 B<date2daysago>,
+B<date2age>,
 B<parse_creator_email>,
 B<adjacent_revision>.
 
@@ -287,6 +303,12 @@ The system functions die on errors.
 
 =over 4
 
+=item B<is_tla_functional>
+
+Verify whether the system has a working arch backend installed (and
+possibly configured by environment variables, like TLA or ARCH_BACKEND),
+needed for this perl library to function.
+
 =item B<run_tla> I<subcommand_with_args>
 
 =item B<run_tla> I<subcommand> arg ...
@@ -363,6 +385,13 @@ If failed, the original string is returned.
 
 Convert a date string to time difference to now in full days.
 
+In list content, return (num_days_ago, unix_time, timezone_str).
+
+=item B<date2age> I<date_string>
+
+Like B<date2daysago>, but return a human readable string, like "5 days"
+or "-6 weeks" or "7 months" or "3 years".
+
 =item B<parse_creator_email> I<my_id>
 
 Try to parse the I<arch> B<my-id> of the patch creator. Return a list of
@@ -385,6 +414,6 @@ Mikhael Goikhman (migo@homemail.com--Perl-GPL/arch-perl--devel).
 
 =head1 SEE ALSO
 
-For more information, see L<tla>, L<Arch::Session>.
+For more information, see L<tla>, L<Arch>.
 
 =cut
